/*
 * Web Designer
 * Copyright (C) 1999-2000  LADISLAS Willhem
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author's email : zillprog@freesurf.fr
 * 
 * snailmail : 
 * LADISLAS Willhem
 * 2, rue du Serpolet
 * 95490 VAUREAL
 * FRANCE
 */

/*
 * This file was written by Gergely Nagy.
 * It contains the functions related to the startup tips.
 */
#include "common_headers.h"
#include "pixmaps/webdesigner-icon.xpm"
#include <time.h>

#ifndef DATADIR
#define DATADIR /usr/local/share
#endif

static struct
{
	GtkWidget *show_tips;
} tips_window;

static char* get_tip ( void );
static char* load_tip ( char *filename );

/*
 * This is the callback function that retieves the
 * values set on the dialog box, and stores them
 * in the preferences struct.
 */
void callback_tips_window ( GtkWidget *widget, gpointer data)
{
#ifdef DEBUG
	fprintf (stderr, "(tips.c) [callback_tips_window] start\n");
#endif
	/*
	 * Get the state of the splash window checkbox...
	 */
	if ( GTK_TOGGLE_BUTTON ( tips_window.show_tips ) -> active )
	{
		preferences.show_tips = TRUE;
	} else {
		preferences.show_tips = FALSE;
	}

	gtk_widget_destroy ( widget );

	global.tips_window_displayed = FALSE;
#ifdef DEBUG
	fprintf (stderr, "(tips.c) [callback_tips_window] end\n");
#endif
}

/*
 * This callback function is used to destroy
 * the current tip window, and display a new one.
 */
void callback_new_tip ( GtkWidget *widget, gpointer data )
{
#ifdef DEBUG
	fprintf (stderr, "(tips.c) [callback_new_tip] start\n");
#endif
	callback_tips_window ( widget, data );
	display_tip_window ();
#ifdef DEBUG
	fprintf (stderr, "(tips.c) [callback_new_tip] end\n");
#endif
}

/*
 * This displays the tip window.
 */
void display_tip_window (void)
{

	GtkWidget *dialog_window;
	GtkWidget *main_vbox;

	GtkWidget *button_box;
	GtkWidget *buttonOk;
	GtkWidget *buttonAnother;

	GtkWidget *misc_box;
	GtkWidget *misc_frame;
	GtkWidget *hbox;

	GdkBitmap *mask;
	GtkWidget *pixmap;
	GdkPixmap *pixm;
	GdkColor color;

	GtkWidget *tip;
	char *tip_text;

#ifdef DEBUG
	fprintf (stderr, "(tips.c) [display_tip_window] start\n");
#endif

	if ( global.tips_window_displayed )
	  {
		return;
	  } else {
		global.tips_window_displayed = TRUE;
	  }

	if ( (tip_text = get_tip ()) == NULL )
	  {
	  	global.tips_window_displayed = FALSE;
		return;
	  }

	/*
	 * Create the dialog window
	 */
	dialog_window = gtk_window_new ( GTK_WINDOW_DIALOG );

	gtk_window_set_title ( GTK_WINDOW ( dialog_window ), _("Startup tips") );
	gtk_widget_set_usize ( dialog_window, 400, 120 );
	gtk_signal_connect ( GTK_OBJECT ( dialog_window ), "destroy", 
			     GTK_SIGNAL_FUNC ( gtk_widget_destroy ), dialog_window);

	gtk_container_set_border_width ( GTK_CONTAINER ( dialog_window ), 5 );

	main_vbox = gtk_vbox_new ( FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER ( dialog_window ), main_vbox );

	/*
	 * Create the frame...
	 */
	misc_frame = gtk_frame_new ( _("Tip") );
	gtk_box_pack_start ( GTK_BOX ( main_vbox ), misc_frame, FALSE, FALSE, 0 );

	misc_box = gtk_vbox_new ( FALSE, 5 );
	gtk_container_add ( GTK_CONTAINER ( misc_frame ), misc_box );

	/*
	 * Display tip...
	 */
	hbox = gtk_hbox_new ( FALSE, 1 );
	gtk_container_add ( GTK_CONTAINER ( misc_box ), hbox );

	tip = gtk_label_new (tip_text);
	gtk_label_set_justify(GTK_LABEL(tip), GTK_JUSTIFY_FILL );
	gtk_label_set_line_wrap(GTK_LABEL(tip), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), tip, FALSE, FALSE, 1);

	gtk_widget_show(dialog_window);
	pixm = gdk_pixmap_create_from_xpm_d( dialog_window->window, &mask, &color, webdesigner_icon_xpm );
	pixmap = gtk_pixmap_new ( pixm, mask );
	gtk_box_pack_end(GTK_BOX(hbox), pixmap, FALSE, FALSE, 1);

	/* 
	 * Create the checkbutton...
	 */
	tips_window.show_tips = gtk_check_button_new_with_label ( _("Show tips at startup") );
	gtk_box_pack_start ( GTK_BOX ( main_vbox ), tips_window.show_tips, FALSE, FALSE, 0 );
	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( tips_window.show_tips ),
				       preferences.show_tips );

	/*
	 * Create the button bar
	 */
	button_box = gtk_hbutton_box_new ();
	gtk_container_set_border_width ( GTK_CONTAINER ( button_box ), 10 );
	gtk_button_box_set_layout ( GTK_BUTTON_BOX ( button_box ), GTK_BUTTONBOX_END );
	gtk_button_box_set_spacing ( GTK_BUTTON_BOX ( button_box ), 10 );
	gtk_button_box_set_child_size ( GTK_BUTTON_BOX ( button_box ), 85, 20 );

	buttonOk = gtk_button_new_with_label ( _("Ok") );
	buttonAnother = gtk_button_new_with_label ( _("Another tip") );

	gtk_container_add ( GTK_CONTAINER ( button_box ), buttonAnother );
	gtk_container_add ( GTK_CONTAINER ( button_box ), buttonOk );

	gtk_container_add ( GTK_CONTAINER ( main_vbox ), button_box );

	gtk_signal_connect_object ( GTK_OBJECT ( buttonOk ), "clicked", 
				    GTK_SIGNAL_FUNC ( callback_tips_window ),
				    GTK_OBJECT ( dialog_window ) );
	gtk_signal_connect_object ( GTK_OBJECT ( buttonAnother ), "clicked", 
				    GTK_SIGNAL_FUNC ( callback_new_tip ),
				    GTK_OBJECT ( dialog_window ) );

	gtk_widget_show_all ( dialog_window );

	free ( tip_text ); 

#ifdef DEBUG
	fprintf (stderr, "(tips.c) [display_tip_window] freed tip_text\n");
	fprintf (stderr, "(tips.c) [display_tip_window] end\n");
#endif
}

/*
 * This one constructs the filename, and loads the tip...
 */
char *get_tip (void)
{
	char *filename;
	char *tip_text;
	char *lang;

#ifdef DEBUG
	fprintf (stderr, "(tips.c) [get_tip] start\n");
#endif

	lang = ( getenv ( "LANG" ) != NULL ) ? getenv ("LANG") : "";

	if ( ( filename = (char *)malloc (strlen(DATADIR) + strlen ("/webdesigner/tips") + strlen ( lang ) + 16 ) ) == NULL )
	  {
	  	merror (_("Not enough memory!"));
#ifdef DEBUG
		fprintf (stderr, "(tips.c) [get_tip] failed to allocate memory for filename\n");
		fprintf (stderr, "(tips.c) [get_tip] end\n");
#endif
		return NULL;
	  }
#ifdef DEBUG
	fprintf (stderr, "(tips.c) [get_tip] allocated %i bytes memory for filename\n", strlen(DATADIR) + strlen ("/webdesigner/tips") + strlen ( lang ) + 16 );
#endif	
	sprintf (filename, "%s/webdesigner/tips.%s", DATADIR, getenv("LANG"));

	if ( (tip_text = load_tip ( filename ) ) == NULL )
	  {
		  sprintf (filename, "%s/webdesigner/tips", DATADIR);
		  if ( ( tip_text = load_tip ( filename ) ) == NULL )
		    {
		    	if ( (tip_text = (char *)malloc (strlen(_("Cannot open tip file!"))+16) ) == NULL )
			  {
				merror ( _("Not enough memory!") );
#ifdef DEBUG
				fprintf (stderr, "(tips.c) [get_tip] failed to allocate memory for tmp\n");
				fprintf (stderr, "(tips.c) [get_tip] end\n");
#endif
				return NULL;
			  }
#ifdef DEBUG
			fprintf (stderr,"(tips.c) [get_tip] allocated %i bytes memory for tmp\n", strlen (_("Cannot open tip file!"))+16);
#endif
		    	sprintf (tip_text, _("Cannot open tip file!"));
		    }
	  }

	free ( filename );
#ifdef DEBUG
	fprintf (stderr, "(tips.c) [get_tip] freed filename\n");
	fprintf (stderr, "(tips.c) [get_tip] end\n");
#endif
	return tip_text;
}

/*
 * This one does the real loading.
 */
char *load_tip ( char *filename ) {
	FILE *tipfile;
	char *tip;
	char *count;
	int tip_count;
	int my_tip;
	int i;

#ifdef DEBUG
	fprintf ( stderr, "(tips.c) [load_tip] start\n");
#endif

	if ( ( tipfile = fopen ( filename, "r" ) ) == NULL )
	{
#ifdef DEBUG
		fprintf (stderr, "(tips.c) [get_tip] file '%s' not found\n", filename );
#endif
		return NULL;
	}

	if ( ( count = (char *)malloc (10) ) == NULL )
	  {
	  	merror ( _("Not enough memory!") );
#ifdef DEBUG
		fprintf (stderr, "(tips.c) [get_tip] failed to allocate memory for count\n");
		fprintf (stderr, "(tips.c) [get_tip] end\n");
#endif
		return NULL;
	  }
#ifdef DEBUG
	fprintf ( stderr, "(tips.c) [load_tip] allocated 10 bytes of memory for count\n");
#endif

	fgets ( count, 5, tipfile );
	tip_count = atoi ( count );

	/*
	 * FIXME :
	 * We'll need a more random thing here...
	 *    - Algernon
	 */
	srand ( time ( NULL ) );
	my_tip = 1 + ( rand() % tip_count );

	/*
	 * We really should have *1* byte of free memory...
	 */
	tip = (char *)malloc(1);
	for ( i = 0; i < my_tip; i++ )
	{
		free ( tip );
		if ( ( tip = get_full_line ( tipfile ) ) == NULL )
		 {
		 	merror ( _("Not enough memory!") );
#ifdef DEBUG
		 	fprintf (stderr, "(tips.c) [get_tip] failed to get tip line. get_full_line returned NULL.\n");
#endif
		 	return NULL;
		 }
	}
	fclose ( tipfile );

	free ( count );
#ifdef DEBUG
	fprintf ( stderr, "(tips.c) [load_tip] freed count\n");
	fprintf ( stderr, "(tips.c) [load_tip] end\n");
#endif
	return tip;
}
